#[repr(transparent)]pub struct cp_layer_renderer_layout(pub u32);layer_renderer_layout only.Expand description
Constants that specify the organization of the textures you use for drawing.
See also Apple’s documentation
Tuple Fields
0: u32Implementations
Sourceimpl cp_layer_renderer_layout
impl cp_layer_renderer_layout
Sourcepub const dedicated: Self
pub const dedicated: Self
A layout that assigns a separate texture to each rendered view.
When the layout contains multiple views, each view receives its own dedicated texture. The type of each texture is MTLTextureType2D.
A layout that uses a single texture to store the content for all rendered views.
When a layer contains multiple views, the texture stores the images for those views side-by-side. The texture map for each view contains a viewport that defines the boundaries of the view’s content. The type of each texture is MTLTextureType2D.
Sourcepub const layered: Self
pub const layered: Self
A layout that specifies each view’s content as a slice of a single texture.
The layout uses a single texture to store the content for all rendered views. The type of the texture is MTLTextureType2DArray. The texture map’s slice index indicates which array slot contains the view’s content.